var MaskHelperAE;
(function (MaskHelperAE) {
    var JsonWriter = (function () {
        function JsonWriter() {
            this.mContent = "";
        }
        JsonWriter.prototype.removeLastComma = function () {
            var last = this.mContent.charAt(this.mContent.length - 1);
            if (last == ",") {
                this.mContent = this.mContent.substr(0, this.mContent.length - 1);
            }
        };
        JsonWriter.prototype.putKey = function (key) {
            this.mContent += "\"" + key + "\":";
        };
        JsonWriter.prototype.putNumber = function (num) {
            this.mContent += num.toString() + "";
            this.mContent += ",";
        };
        JsonWriter.prototype.putString = function (str) {
            this.mContent += JSON.stringify(str);
            this.mContent += ",";
        };
        JsonWriter.prototype.putPureString = function (str) {
            this.mContent += "\"" + str + "\"";
            this.mContent += ",";
        };
        JsonWriter.prototype.putBoolean = function (bool) {
            this.mContent += bool.toString();
            this.mContent += ",";
        };
        JsonWriter.prototype.startArray = function () {
            this.mContent += "[";
        };
        JsonWriter.prototype.endArray = function () {
            this.removeLastComma();
            this.mContent += "],";
        };
        JsonWriter.prototype.startObject = function () {
            this.mContent += "{";
        };
        JsonWriter.prototype.endObject = function () {
            this.removeLastComma();
            this.mContent += "},";
        };
        JsonWriter.prototype.getContent = function () {
            var last = this.mContent.charAt(this.mContent.length - 1);
            if (last == ",") {
                return this.mContent.substr(0, this.mContent.length - 1);
            }
            else {
                return this.mContent;
            }
        };
        JsonWriter.prototype.writeToFile = function (path) {
            var file = new File(path);
            file.open("w");
            file.encoding = "utf-8";
            file.write(this.mContent);
            file.close();
        };
        return JsonWriter;
    }());
    var DomUtils = (function () {
        function DomUtils() {
        }
        DomUtils.GetCurrentItem = function () {
            var proj = app.project;
            if (proj != null) {
                var comp = proj.activeItem;
                return comp;
            }
            return null;
        };
        DomUtils.GetCurrentComp = function () {
            var item = this.GetCurrentItem();
            if (item && item instanceof CompItem) {
                return item;
            }
            return null;
        };
        DomUtils.GetSelectedLayers = function () {
            var comp = this.GetCurrentComp();
            if (comp) {
                return comp.selectedLayers;
            }
            return null;
        };
        DomUtils.GetFirstSelectedLayer = function () {
            var layers = this.GetSelectedLayers();
            if (layers && layers.length > 0) {
                return layers[0];
            }
            return null;
        };
        DomUtils.GetLayersInComp = function (comp) {
            var layers = [];
            for (var i = 1; i <= comp.numLayers; i++) {
                layers.push(comp.layer(i));
            }
            return layers;
        };
        DomUtils.FindCompByName = function (name) {
            var proj = app.project;
            if (proj != null) {
                for (var i = proj.numItems; i > 0; i--) {
                    var comp = proj.item(i);
                    if (comp instanceof CompItem && comp.name == name) {
                        return comp;
                    }
                }
            }
            return null;
        };
        DomUtils.GetItemName = function (index) {
            var proj = app.project;
            if (proj != null) {
                if (index >= 1 && index <= proj.numItems) {
                    return proj.item(index).name;
                }
            }
            return null;
        };
        DomUtils.GetItemById = function (id) {
            var proj = app.project;
            if (proj != null) {
                for (var i = proj.numItems; i > 0; i--) {
                    if (proj.item(i).id == id) {
                        return proj.item(i);
                    }
                }
            }
            return null;
        };
        DomUtils.GetCompByIndex = function (index) {
            var proj = app.project;
            if (proj != null) {
                if (index >= 1 && index <= proj.numItems) {
                    var comp = proj.item(index);
                    if (comp instanceof CompItem) {
                        return comp;
                    }
                }
            }
            return null;
        };
        DomUtils.GetCompIndexById = function (id) {
            var proj = app.project;
            if (proj != null) {
                for (var i = proj.numItems; i > 0; i--) {
                    var comp = proj.item(i);
                    if (comp instanceof CompItem && comp.id == id) {
                        return i;
                    }
                }
            }
            return null;
        };
        DomUtils.GetCompLayerName = function (compIndex, layer) {
            var proj = app.project;
            if (proj != null) {
                if (compIndex >= 1 && compIndex <= proj.numItems) {
                    var comp = proj.item(compIndex);
                    if (comp instanceof CompItem && layer >= 1 && layer <= comp.numLayers) {
                        return comp.layer(layer).name;
                    }
                }
            }
            return null;
        };
        DomUtils.RemoveLayerDisabledEffects = function (layer) {
            var count = 0;
            var e = layer.effect;
            for (var i = e.numProperties; i > 0; i--) {
                var f = e.property(i);
                if (!f.active) {
                    count++;
                    f.remove();
                    i--;
                }
            }
            return count;
        };
        DomUtils.RemoveLayerAllEffects = function (layer) {
            var e = layer.effect;
            for (var i = e.numProperties; i > 0; i--) {
                var f = e.property(i);
                f.remove();
                i--;
            }
        };
        DomUtils.IsLayerReallyHasEffects = function (layer) {
            if (layer.effect.numProperties > 0 && layer.effectsActive) {
                for (var ee = 1; ee <= layer.effect.numProperties; ee++) {
                    if (layer.effect.property(ee).enabled) {
                        return true;
                    }
                }
            }
            return false;
        };
        DomUtils.RemoveLayerAllMasks = function (layer) {
            var e = layer.mask;
            for (var i = e.numProperties; i > 0; i--) {
                var f = e.property(i);
                f.remove();
                i--;
            }
        };
        DomUtils.GetCamerasLayerInComp = function (comp) {
            var cameras = [];
            for (var i = 1, l = comp.numLayers; i <= l; i++) {
                var layer = comp.layer(i);
                if (layer instanceof CameraLayer && layer.enabled) {
                    cameras.push(layer);
                }
            }
            return cameras;
        };
        DomUtils.GetLightLayerInComp = function (comp) {
            var lights = [];
            for (var i = 1, l = comp.numLayers; i <= l; i++) {
                var layer = comp.layer(i);
                if (layer instanceof LightLayer) {
                    lights.push(layer);
                }
            }
            return lights;
        };
        DomUtils.AddDefaultCameraToComp = function (comp) {
            var layer = comp.layers.addCamera("Default Camera", [comp.width * 0.5, comp.height * 0.5]);
            var p = layer.position.value;
            layer.position.setValue([comp.width * 0.5, comp.height * 0.5, p[2]]);
            layer.pointOfInterest.setValue([comp.width * 0.5, comp.height * 0.5, 0]);
            layer.moveToEnd();
        };
        DomUtils.ReplacePrecompWithFile = function (oldItem, newFile) {
            var impOpt = new ImportOptions(newFile);
            var footageItem = app.project.importFile(impOpt);
            var comps = oldItem.usedIn;
            for (var c = 0; c < comps.length; c++) {
                var comp = comps[c];
                for (var l = 1; l <= comp.numLayers; l++) {
                    var layer = comp.layer(l);
                    if (layer instanceof AVLayer) {
                        if (layer.source.id == oldItem.id) {
                            layer.source.replaceSource(footageItem);
                        }
                    }
                }
            }
        };
        DomUtils.ReplaceFootageItem = function (oldFootage, newFile) {
            if (oldFootage) {
                oldFootage.replace(newFile);
            }
        };
        DomUtils.IsPrecompInsideMainComp = function (precomp, mainComp) {
            if (precomp.id == mainComp.id) {
                return true;
            }
            for (var i = 1; i <= mainComp.numLayers; i++) {
                var layer = mainComp.layer(i);
                if (layer instanceof AVLayer && layer.source instanceof CompItem) {
                    if (layer.source.id == precomp.id || DomUtils.IsPrecompInsideMainComp(precomp, layer.source)) {
                        return true;
                    }
                }
            }
            return false;
        };
        DomUtils.DisableCompAudio = function (comp) {
            var numLayers = comp.numLayers;
            for (var l = 1; l <= numLayers; l++) {
                var layer = comp.layer(l);
                if (layer instanceof AVLayer) {
                    layer.audioEnabled = false;
                }
            }
        };
        DomUtils.ShowItem = function (id) {
            var proj = app.project;
            if (proj != null) {
                for (var i = 1; i <= proj.numItems; i++) {
                    if (app.project.item(i).id == id) {
                        app.project.item(i).selected = true;
                    }
                    else {
                        app.project.item(i).selected = false;
                    }
                }
            }
        };
        return DomUtils;
    }());
    var LayerTypes;
    (function (LayerTypes) {
        LayerTypes[LayerTypes["kAVLayer"] = 1] = "kAVLayer";
        LayerTypes[LayerTypes["kCameraLayer"] = 2] = "kCameraLayer";
        LayerTypes[LayerTypes["kLightLayer"] = 4] = "kLightLayer";
        LayerTypes[LayerTypes["kShapeLayer"] = 8] = "kShapeLayer";
        LayerTypes[LayerTypes["kTextLayer"] = 16] = "kTextLayer";
        LayerTypes[LayerTypes["kPlaceholderStillLayer"] = 32] = "kPlaceholderStillLayer";
        LayerTypes[LayerTypes["kSolidLayer"] = 64] = "kSolidLayer";
        LayerTypes[LayerTypes["kStillLayer"] = 128] = "kStillLayer";
        LayerTypes[LayerTypes["kGuideLayer"] = 256] = "kGuideLayer";
        LayerTypes[LayerTypes["kAdjustmentLayer"] = 512] = "kAdjustmentLayer";
        LayerTypes[LayerTypes["kNullLayer"] = 1024] = "kNullLayer";
        LayerTypes[LayerTypes["kAudioLayer"] = 2048] = "kAudioLayer";
        LayerTypes[LayerTypes["kPreCompLayer"] = 4096] = "kPreCompLayer";
        LayerTypes[LayerTypes["kPlaceholderVideoLayer"] = 8192] = "kPlaceholderVideoLayer";
        LayerTypes[LayerTypes["kImageSequenceLayer"] = 16384] = "kImageSequenceLayer";
        LayerTypes[LayerTypes["kVideoLayer"] = 32768] = "kVideoLayer";
        LayerTypes[LayerTypes["kThreeDLayer"] = 65536] = "kThreeDLayer";
    })(LayerTypes || (LayerTypes = {}));
    var LayerTypeUtils = (function () {
        function LayerTypeUtils() {
        }
        LayerTypeUtils.GetLayerType = function (layer) {
            function avLayerType(lObj) {
                var lSource = lObj.source;
                if (lSource instanceof CompItem) {
                    return LayerTypes.kPreCompLayer;
                }
                var lMainSource = lSource.mainSource;
                var lFile = lMainSource.file;
                if (!lObj.hasVideo) {
                    return LayerTypes.kAudioLayer;
                }
                else if (lSource instanceof CompItem) {
                    return LayerTypes.kPreCompLayer;
                }
                else if (lSource.frameDuration < 1) {
                    if (lMainSource instanceof PlaceholderSource) {
                        return LayerTypes.kPlaceholderVideoLayer;
                    }
                    else if (lSource.name.toString().indexOf("].") != -1) {
                        return LayerTypes.kImageSequenceLayer;
                    }
                    else {
                        return LayerTypes.kVideoLayer;
                    }
                }
                else {
                    if (lSource.frameDuration == 1) {
                        if (lMainSource instanceof PlaceholderSource) {
                            return LayerTypes.kPlaceholderStillLayer;
                        }
                        else if (lMainSource.color) {
                            return LayerTypes.kSolidLayer;
                        }
                        else {
                            return LayerTypes.kStillLayer;
                        }
                    }
                }
                return 0;
            }
            try {
                var result = 0;
                var curLayer = layer;
                var instanceOfArray = [AVLayer, CameraLayer, LightLayer, ShapeLayer, TextLayer];
                var instanceOfArrayLength = instanceOfArray.length;
                var avLayer = (LayerTypeUtils.IsBaseAVLayer(curLayer)) ? curLayer : null;
                if (curLayer.nullLayer) {
                    if (avLayer) {
                        return LayerTypes.kNullLayer | (avLayer.threeDLayer ? LayerTypes.kThreeDLayer : 0);
                    }
                    else {
                        return LayerTypes.kNullLayer;
                    }
                }
                else if (avLayer) {
                    if (avLayer.guideLayer) {
                        return LayerTypes.kGuideLayer | (avLayer.threeDLayer ? LayerTypes.kThreeDLayer : 0);
                    }
                    else if (avLayer.adjustmentLayer) {
                        return LayerTypes.kAdjustmentLayer | (avLayer.threeDLayer ? LayerTypes.kThreeDLayer : 0);
                    }
                }
                for (var i = 0; i < instanceOfArrayLength; i += 1) {
                    if (curLayer instanceof instanceOfArray[i]) {
                        result = 1 << i;
                        break;
                    }
                }
                if (result & LayerTypes.kAVLayer) {
                    result |= avLayerType(curLayer);
                    result |= curLayer.threeDLayer ? LayerTypes.kThreeDLayer : 0;
                }
                return result;
            }
            catch (err) {
                alert(err.line.toString + " " + err.toString());
            }
            return 0;
        };
        LayerTypeUtils.IsAVLayer = function (layer) {
            return layer instanceof AVLayer;
        };
        LayerTypeUtils.IsBaseAVLayer = function (layer) {
            return layer instanceof AVLayer || layer instanceof TextLayer || layer instanceof ShapeLayer;
        };
        LayerTypeUtils.GetUIString = function (type) {
            if (type & LayerTypes.kAudioLayer) {
                return "Audio Layer";
            }
            else if (type & LayerTypes.kPreCompLayer) {
                return "PreComp Layer";
            }
            else if (type & LayerTypes.kPlaceholderVideoLayer) {
                return "Placeholder Video Layer";
            }
            else if (type & LayerTypes.kImageSequenceLayer) {
                return "Image Sequence Layer";
            }
            else if (type & LayerTypes.kVideoLayer) {
                return "Video Layer";
            }
            else if (type & LayerTypes.kPlaceholderStillLayer) {
                return "Placeholder Still Layer";
            }
            else if (type & LayerTypes.kSolidLayer) {
                return "Solid Layer";
            }
            else if (type & LayerTypes.kStillLayer) {
                return "Still Layer";
            }
            else if (type & LayerTypes.kGuideLayer) {
                return "Guide Layer";
            }
            else if (type & LayerTypes.kAdjustmentLayer) {
                return "Adjustment Layer";
            }
            else if (type & LayerTypes.kNullLayer) {
                return "Null Layer";
            }
            else if (type & LayerTypes.kAVLayer) {
                return "AV Layer";
            }
            else if (type & LayerTypes.kCameraLayer) {
                return "Camera Layer";
            }
            else if (type & LayerTypes.kLightLayer) {
                return "Light Layer";
            }
            else if (type & LayerTypes.kShapeLayer) {
                return "Shape Layer";
            }
            else if (type & LayerTypes.kTextLayer) {
                return "Text Layer";
            }
            else {
                return "";
            }
        };
        return LayerTypeUtils;
    }());
    var PointInfo = (function () {
        function PointInfo(_x, _y) {
            this.mX = _x;
            this.mY = _y;
        }
        PointInfo.prototype.equal = function (other) {
            return Math.pow(this.mX - other.mX, 2.0) + Math.pow(this.mY - other.mY, 2.0) < 0.5;
        };
        PointInfo.prototype.toJson = function (writer) {
            writer.startArray();
            writer.putNumber(Math.round(this.mX));
            writer.putNumber(Math.round(this.mY));
            writer.endArray();
        };
        PointInfo.prototype.fromJson = function (data) {
            if (data instanceof Array && data.length == 2) {
                this.mX = data[0];
                this.mY = data[1];
            }
        };
        return PointInfo;
    }());
    var PathInfo = (function () {
        function PathInfo() {
            this.mHandle1 = null;
            this.mHandle2 = null;
        }
        PathInfo.prototype.toJson = function (writer) {
            writer.startArray();
            this.mPoint1.toJson(writer);
            this.mPoint2.toJson(writer);
            if (this.mHandle1) {
                this.mHandle1.toJson(writer);
            }
            if (this.mHandle2) {
                this.mHandle2.toJson(writer);
            }
            writer.endArray();
        };
        PathInfo.prototype.fromJson = function (data) {
            if (data instanceof Array && (data.length == 2 || data.length == 4)) {
                this.mPoint1 = new PointInfo(0, 0);
                this.mPoint1.fromJson(data[0]);
                this.mPoint2 = new PointInfo(0, 0);
                this.mPoint2.fromJson(data[1]);
                if (data.length == 4) {
                    this.mHandle1 = new PointInfo(0, 0);
                    this.mHandle1.fromJson(data[2]);
                    this.mHandle2 = new PointInfo(0, 0);
                    this.mHandle2.fromJson(data[3]);
                }
            }
        };
        return PathInfo;
    }());
    var ShapeMode;
    (function (ShapeMode) {
        ShapeMode[ShapeMode["kAddMode"] = 0] = "kAddMode";
        ShapeMode[ShapeMode["kSubtractMode"] = 1] = "kSubtractMode";
        ShapeMode[ShapeMode["kIntersectMode"] = 2] = "kIntersectMode";
        ShapeMode[ShapeMode["kDifferenceMode"] = 3] = "kDifferenceMode";
        ShapeMode[ShapeMode["kNoneMode"] = 4] = "kNoneMode";
    })(ShapeMode || (ShapeMode = {}));
    var ShapeInfo = (function () {
        function ShapeInfo() {
            this.mIndex = -1;
            this.mInverted = false;
            this.mPaths = [];
            this.mClosed = true;
            this.mMode = ShapeMode.kAddMode;
        }
        ShapeInfo.prototype.toJson = function (writer) {
            writer.startObject();
            writer.putKey("index");
            writer.putNumber(this.mIndex);
            writer.putKey("invert");
            writer.putBoolean(this.mInverted);
            writer.putKey("mode");
            writer.putNumber(this.mMode);
            writer.putKey("closed");
            writer.putBoolean(this.mClosed);
            writer.putKey("paths");
            writer.startArray();
            for (var p = 0; p < this.mPaths.length; p++) {
                this.mPaths[p].toJson(writer);
            }
            writer.endArray();
            writer.endObject();
        };
        ShapeInfo.prototype.fromJson = function (data) {
            this.mPaths = [];
            this.mClosed = true;
            this.mInverted = false;
            this.mMode = ShapeMode.kAddMode;
            var index = data.index;
            if (typeof (index) != "undefined") {
                this.mIndex = index;
            }
            var closed = data.closed;
            if (typeof (closed) != "undefined") {
                this.mClosed = closed;
            }
            var invert = data.invert;
            if (typeof (invert) != "undefined") {
                this.mInverted = invert;
            }
            var mode = data.mode;
            if (typeof (mode) != "undefined") {
                this.mMode = mode;
            }
            var paths = data.paths;
            if (paths instanceof Array) {
                for (var i = 0; i < paths.length; i++) {
                    var path = new PathInfo();
                    path.fromJson(paths[i]);
                    this.mPaths.push(path);
                }
            }
        };
        return ShapeInfo;
    }());
    var ResultInfo = (function () {
        function ResultInfo(x, y, w, h) {
            this.mOriginX = x;
            this.mOriginY = y;
            this.mWidth = w;
            this.mHeight = h;
            this.mIndices = [];
            this.clear();
        }
        ResultInfo.prototype.clear = function () {
            this.mShapes = [];
            this.mIndices = [];
            this.mLastPoint = null;
            this.mLastHandle = null;
        };
        ResultInfo.prototype.moveTo = function (x, y) {
            var shape = new ShapeInfo();
            this.mShapes.push(shape);
            this.mLastPoint = new PointInfo(x, y);
            this.mLastHandle = null;
        };
        ResultInfo.prototype.lineTo = function (x, y) {
            if (this.mShapes.length > 0) {
                if (this.mLastPoint) {
                    var path = new PathInfo();
                    path.mPoint1 = this.mLastPoint;
                    path.mPoint2 = new PointInfo(x, y);
                    this.mShapes[this.mShapes.length - 1].mPaths.push(path);
                }
                this.mLastPoint = new PointInfo(x, y);
                this.mLastHandle = null;
            }
        };
        ResultInfo.prototype.bezierTo = function (x, y, cx1, cy1, cx2, cy2) {
            if (this.mShapes.length > 0) {
                if (this.mLastPoint) {
                    var path = new PathInfo();
                    path.mPoint1 = this.mLastPoint;
                    path.mPoint2 = new PointInfo(x, y);
                    path.mHandle1 = new PointInfo(cx1, cy1);
                    path.mHandle2 = new PointInfo(cx2, cy2);
                    this.mShapes[this.mShapes.length - 1].mPaths.push(path);
                }
                this.mLastPoint = new PointInfo(x, y);
                this.mLastHandle = null;
            }
        };
        ResultInfo.prototype.lastShape = function () {
            if (this.mShapes.length > 0) {
                return this.mShapes[this.mShapes.length - 1];
            }
            return null;
        };
        ResultInfo.prototype.toJson = function () {
            var writer = new JsonWriter();
            writer.startObject();
            writer.putKey("shapes");
            writer.startArray();
            for (var s = 0, l = this.mShapes.length; s < l; s++) {
                this.mShapes[s].toJson(writer);
            }
            writer.endArray();
            writer.putKey("indices");
            writer.startArray();
            for (var s = 0, l = this.mIndices.length; s < l; s++) {
                writer.putNumber(this.mIndices[s]);
            }
            writer.endArray();
            writer.putKey("x");
            writer.putNumber(this.mOriginX);
            writer.putKey("y");
            writer.putNumber(this.mOriginY);
            writer.putKey("width");
            writer.putNumber(this.mWidth);
            writer.putKey("height");
            writer.putNumber(this.mHeight);
            writer.endObject();
            return writer.getContent();
        };
        ResultInfo.prototype.fromJson = function (json) {
            this.clear();
            var data = JSON.parse(json);
            var shapes = data.shapes;
            if (shapes instanceof Array) {
                for (var s = 0; s < shapes.length; s++) {
                    var shape = new ShapeInfo();
                    shape.fromJson(shapes[s]);
                    this.mShapes.push(shape);
                }
            }
            var indices = data.indices;
            if (typeof (indices) != "undefined") {
                this.mIndices = indices;
            }
            var x = data.x;
            if (typeof (x) != "undefined") {
                this.mOriginX = x;
            }
            var y = data.y;
            if (typeof (y) != "undefined") {
                this.mOriginY = y;
            }
            var width = data.width;
            if (typeof (width) != "undefined") {
                this.mWidth = width;
            }
            var height = data.height;
            if (typeof (height) != "undefined") {
                this.mHeight = height;
            }
        };
        return ResultInfo;
    }());
    var MaskExporter = (function () {
        function MaskExporter() {
        }
        MaskExporter.ConvertMaskModeToShapeMode = function (mode) {
            switch (mode) {
                case MaskMode.NONE:
                    return ShapeMode.kNoneMode;
                case MaskMode.DARKEN:
                case MaskMode.LIGHTEN:
                    return null;
                case MaskMode.ADD:
                    return ShapeMode.kAddMode;
                case MaskMode.SUBTRACT:
                    return ShapeMode.kSubtractMode;
                case MaskMode.INTERSECT:
                    return ShapeMode.kIntersectMode;
                case MaskMode.DIFFERENCE:
                    return ShapeMode.kDifferenceMode;
                default:
                    return null;
            }
        };
        MaskExporter.HasShapeAnimation = function (mask) {
            return mask.maskShape.numKeys > 1 || (mask.maskShape.expressionEnabled && mask.maskShape.expression.length > 0);
        };
        MaskExporter.ExportMaskValue = function (mask, result, acceptOpenPaths) {
            if (acceptOpenPaths === void 0) { acceptOpenPaths = false; }
            var mode = MaskExporter.ConvertMaskModeToShapeMode(mask.maskMode);
            mode = acceptOpenPaths ? (mode == null ? ShapeMode.kAddMode : mode) : mode;
            if (mode != null) {
                var shapeObj = mask.maskShape.value;
                if (shapeObj.vertices.length >= 3 && (shapeObj.closed || acceptOpenPaths)) {
                    var poly = 0;
                    var vertices = shapeObj.vertices;
                    var inTangents = shapeObj.inTangents;
                    var outTangents = shapeObj.outTangents;
                    for (var i_1 = 0; i_1 < vertices.length; i_1++) {
                        inTangents[i_1] = [inTangents[i_1][0] + vertices[i_1][0], inTangents[i_1][1] + vertices[i_1][1]];
                        outTangents[i_1] = [outTangents[i_1][0] + vertices[i_1][0], outTangents[i_1][1] + vertices[i_1][1]];
                        if (inTangents[i_1][0] == vertices[i_1][0] && inTangents[i_1][1] == vertices[i_1][1] && outTangents[i_1][0] == vertices[i_1][0] && outTangents[i_1][1] == vertices[i_1][1]) {
                            poly += 0;
                        }
                        else {
                            poly += 1;
                        }
                    }
                    ;
                    if (poly == 0) {
                        result.moveTo(vertices[0][0], vertices[0][1]);
                        for (var i = 1; i < vertices.length; i++) {
                            result.lineTo(vertices[i][0], vertices[i][1]);
                        }
                        ;
                        result.lineTo(vertices[0][0], vertices[0][1]);
                    }
                    else {
                        result.moveTo(vertices[0][0], vertices[0][1]);
                        for (var i = 1; i < vertices.length; i++) {
                            if (outTangents[i - 1][0] == vertices[i - 1][0] && outTangents[i - 1][1] == vertices[i - 1][1] && inTangents[i][0] == vertices[i][0] && inTangents[i][1] == vertices[i][1]) {
                                result.lineTo(vertices[i][0], vertices[i][1]);
                            }
                            else {
                                result.bezierTo(vertices[i][0], vertices[i][1], outTangents[i - 1][0], outTangents[i - 1][1], inTangents[i][0], inTangents[i][1]);
                            }
                        }
                        ;
                        if (outTangents[outTangents.length - 1][0] == vertices[outTangents.length - 1][0] && outTangents[outTangents.length - 1][1] == vertices[outTangents.length - 1][1] && inTangents[0][0] == vertices[0][1] && inTangents[0][0] == vertices[0][1]) {
                            result.lineTo(vertices[0][0], vertices[0][1]);
                        }
                        else {
                            result.bezierTo(vertices[0][0], vertices[0][1], outTangents[outTangents.length - 1][0], outTangents[outTangents.length - 1][1], inTangents[0][0], inTangents[0][1]);
                        }
                    }
                    result.lastShape().mClosed = shapeObj.closed;
                    result.lastShape().mMode = mode;
                    result.lastShape().mInverted = mask.inverted;
                    result.lastShape().mIndex = mask.propertyIndex;
                    result.mIndices.push(mask.propertyIndex);
                    return true;
                }
            }
            return false;
        };
        MaskExporter.ExportLayerMasks = function (layer, acceptOpenPaths) {
            if (acceptOpenPaths === void 0) { acceptOpenPaths = false; }
            if (layer.mask.numProperties > 0) {
                var result = new ResultInfo(0, 0, layer.width, layer.height);
                for (var p = 0, l = layer.mask.numProperties; p < l; p++) {
                    MaskExporter.ExportMaskValue(layer.mask.property(p + 1), result, acceptOpenPaths);
                }
                return result;
            }
            return null;
        };
        MaskExporter.SelectedMaskHasAnimation = function (layer) {
            if (layer.mask.numProperties > 0) {
                for (var p = 0, l = layer.mask.numProperties; p < l; p++) {
                    if (layer.mask.property(p + 1).selected) {
                        if (MaskExporter.HasShapeAnimation(layer.mask.property(p + 1))) {
                            return true;
                        }
                    }
                }
                return false;
            }
            return false;
        };
        MaskExporter.ExportLayerSelectedMasks = function (layer, acceptOpenPaths) {
            if (acceptOpenPaths === void 0) { acceptOpenPaths = false; }
            if (layer.mask.numProperties > 0) {
                var result = new ResultInfo(0, 0, layer.width, layer.height);
                for (var p = 0, l = layer.mask.numProperties; p < l; p++) {
                    if (layer.mask.property(p + 1).selected) {
                        MaskExporter.ExportMaskValue(layer.mask.property(p + 1), result, acceptOpenPaths);
                    }
                }
                return result;
            }
            return null;
        };
        return MaskExporter;
    }());
    var MaskHandler = (function () {
        function MaskHandler() {
        }
        MaskHandler.Handle = function (result) {
            var firstLayer = DomUtils.GetFirstSelectedLayer();
            if (firstLayer && LayerTypeUtils.IsBaseAVLayer(firstLayer)) {
                var avLayer = firstLayer;
                if (result.mShapes.length > 0) {
                    for (var i = 0; i < result.mIndices.length; i++) {
                        avLayer.mask.property(result.mIndices[i] - i).remove();
                    }
                }
                for (var s = 0; s < result.mShapes.length; s++) {
                    var ret = MaskHandler.AddShapeToLayer(result.mShapes[s], avLayer);
                    if (ret) {
                        ret.selected = true;
                        ret.moveTo(result.mShapes[s].mIndex);
                    }
                }
                return true;
            }
            return false;
        };
        MaskHandler.AddShapeToLayer = function (shape, layer) {
            if (shape.mPaths.length >= 3) {
                var mask = layer.mask.addProperty("ADBE Mask Atom");
                var shapeObj = mask.maskShape.value;
                shapeObj.closed = shape.mClosed;
                var vertices = [];
                var outTangents = [];
                var inTangents = [];
                for (var p = 0; p < shape.mPaths.length; p++) {
                    var path = shape.mPaths[p];
                    vertices.push([path.mPoint1.mX, path.mPoint1.mY]);
                    outTangents.push([0, 0]);
                    inTangents.push([0, 0]);
                }
                shapeObj.vertices = vertices;
                for (var p = 0; p < shape.mPaths.length; p++) {
                    var path = shape.mPaths[p];
                    if (path.mHandle1 != null) {
                        outTangents[p][0] = path.mHandle1.mX - path.mPoint1.mX;
                        outTangents[p][1] = path.mHandle1.mY - path.mPoint1.mY;
                    }
                    if (path.mHandle2 != null) {
                        if (p == shape.mPaths.length - 1) {
                            inTangents[0][0] = path.mHandle2.mX - shape.mPaths[0].mPoint1.mX;
                            inTangents[0][1] = path.mHandle2.mY - shape.mPaths[0].mPoint1.mY;
                        }
                        else {
                            inTangents[p + 1][0] = path.mHandle2.mX - shape.mPaths[p + 1].mPoint1.mX;
                            inTangents[p + 1][1] = path.mHandle2.mY - shape.mPaths[p + 1].mPoint1.mY;
                        }
                    }
                }
                shapeObj.outTangents = outTangents;
                shapeObj.inTangents = inTangents;
                switch (shape.mMode) {
                    case ShapeMode.kNoneMode:
                        mask.maskMode = MaskMode.NONE;
                        break;
                    case ShapeMode.kAddMode:
                        mask.maskMode = MaskMode.ADD;
                        break;
                    case ShapeMode.kDifferenceMode:
                        mask.maskMode = MaskMode.DIFFERENCE;
                        break;
                    case ShapeMode.kIntersectMode:
                        mask.maskMode = MaskMode.INTERSECT;
                        break;
                    case ShapeMode.kSubtractMode:
                        mask.maskMode = MaskMode.SUBTRACT;
                        break;
                    default:
                        break;
                }
                mask.maskShape.setValue(shapeObj);
                return mask;
            }
            return null;
        };
        return MaskHandler;
    }());
    function IsSecurityPrefSet() {
        var securitySetting = app.preferences.getPrefAsLong("Main Pref Section", "Pref_SCRIPTING_FILE_NETWORK_SECURITY");
        return (securitySetting == 1);
    }
    MaskHelperAE.IsSecurityPrefSet = IsSecurityPrefSet;
    function FetchLayerAllMasks(option) {
        var firstLayer = DomUtils.GetFirstSelectedLayer();
        if (firstLayer && LayerTypeUtils.IsBaseAVLayer(firstLayer)) {
            var result = MaskExporter.ExportLayerMasks(firstLayer, option == "acceptOpenPaths");
            if (result == null) {
                return "\{\"error\":\"no masks\"\}";
            }
            else {
                return result.toJson();
            }
        }
        else {
            return "\{\"error\":\"no layer\"\}";
        }
    }
    MaskHelperAE.FetchLayerAllMasks = FetchLayerAllMasks;
    function FetchLayerSelectedMasks(option) {
        var firstLayer = DomUtils.GetFirstSelectedLayer();
        if (firstLayer && LayerTypeUtils.IsBaseAVLayer(firstLayer)) {
            var animated = MaskExporter.SelectedMaskHasAnimation(firstLayer);
            if (animated) {
                return "\{\"error\":\"animated\"\}";
            }
            else {
                var result = MaskExporter.ExportLayerSelectedMasks(firstLayer, option == "acceptOpenPaths");
                if (result == null) {
                    return "\{\"error\":\"no masks\"\}";
                }
                else {
                    return result.toJson();
                }
            }
        }
        else {
            return "\{\"error\":\"no layer\"\}";
        }
    }
    MaskHelperAE.FetchLayerSelectedMasks = FetchLayerSelectedMasks;
    function GenerateShapeResult(result) {
        var ret = false;
        var resultInfo = new ResultInfo(0, 0, 0, 0);
        resultInfo.fromJson(result);
        app.beginUndoGroup("VE Mask Helper");
        ret = MaskHandler.Handle(resultInfo);
        app.endUndoGroup();
        return ret;
    }
    MaskHelperAE.GenerateShapeResult = GenerateShapeResult;
    function RemoveFile(path) {
        var file = new File(path);
        if (file.exists) {
            file.remove();
        }
    }
    MaskHelperAE.RemoveFile = RemoveFile;
    function Alert(msg, title, errIcon) {
        if (title === void 0) { title = null; }
        if (errIcon === void 0) { errIcon = false; }
        alert(msg, title, errIcon);
    }
    MaskHelperAE.Alert = Alert;
})(MaskHelperAE || (MaskHelperAE = {}));
