"use strict";
var MaskHelper;
(function (MaskHelper) {
    var JsonWriter = (function () {
        function JsonWriter() {
            this.mContent = "";
        }
        JsonWriter.prototype.removeLastComma = function () {
            var last = this.mContent.charAt(this.mContent.length - 1);
            if (last == ",") {
                this.mContent = this.mContent.substr(0, this.mContent.length - 1);
            }
        };
        JsonWriter.prototype.putKey = function (key) {
            this.mContent += "\"" + key + "\":";
        };
        JsonWriter.prototype.putNumber = function (num) {
            this.mContent += num.toString() + "";
            this.mContent += ",";
        };
        JsonWriter.prototype.putString = function (str) {
            this.mContent += JSON.stringify(str);
            this.mContent += ",";
        };
        JsonWriter.prototype.putPureString = function (str) {
            this.mContent += "\"" + str + "\"";
            this.mContent += ",";
        };
        JsonWriter.prototype.putBoolean = function (bool) {
            this.mContent += bool.toString();
            this.mContent += ",";
        };
        JsonWriter.prototype.startArray = function () {
            this.mContent += "[";
        };
        JsonWriter.prototype.endArray = function () {
            this.removeLastComma();
            this.mContent += "],";
        };
        JsonWriter.prototype.startObject = function () {
            this.mContent += "{";
        };
        JsonWriter.prototype.endObject = function () {
            this.removeLastComma();
            this.mContent += "},";
        };
        JsonWriter.prototype.getContent = function () {
            var last = this.mContent.charAt(this.mContent.length - 1);
            if (last == ",") {
                return this.mContent.substr(0, this.mContent.length - 1);
            }
            else {
                return this.mContent;
            }
        };
        return JsonWriter;
    }());
    var PointInfo = (function () {
        function PointInfo(_x, _y) {
            this.mX = _x;
            this.mY = _y;
        }
        PointInfo.prototype.equal = function (other) {
            return Math.pow(this.mX - other.mX, 2.0) + Math.pow(this.mY - other.mY, 2.0) < 0.001;
        };
        PointInfo.prototype.toJson = function (writer) {
            writer.startArray();
            writer.putNumber(Math.round(this.mX));
            writer.putNumber(Math.round(this.mY));
            writer.endArray();
        };
        PointInfo.prototype.fromJson = function (data) {
            if (data instanceof Array && data.length == 2) {
                this.mX = data[0];
                this.mY = data[1];
            }
        };
        return PointInfo;
    }());
    var PathInfo = (function () {
        function PathInfo() {
            this.mHandle1 = null;
            this.mHandle2 = null;
        }
        PathInfo.prototype.toJson = function (writer) {
            writer.startArray();
            this.mPoint1.toJson(writer);
            this.mPoint2.toJson(writer);
            if (this.mHandle1) {
                this.mHandle1.toJson(writer);
            }
            if (this.mHandle2) {
                this.mHandle2.toJson(writer);
            }
            writer.endArray();
        };
        PathInfo.prototype.fromJson = function (data) {
            if (data instanceof Array && (data.length == 2 || data.length == 4)) {
                this.mPoint1 = new PointInfo(0, 0);
                this.mPoint1.fromJson(data[0]);
                this.mPoint2 = new PointInfo(0, 0);
                this.mPoint2.fromJson(data[1]);
                if (data.length == 4) {
                    this.mHandle1 = new PointInfo(0, 0);
                    this.mHandle1.fromJson(data[2]);
                    this.mHandle2 = new PointInfo(0, 0);
                    this.mHandle2.fromJson(data[3]);
                }
            }
        };
        return PathInfo;
    }());
    var ShapeMode;
    (function (ShapeMode) {
        ShapeMode[ShapeMode["kAddMode"] = 0] = "kAddMode";
        ShapeMode[ShapeMode["kSubtractMode"] = 1] = "kSubtractMode";
        ShapeMode[ShapeMode["kIntersectMode"] = 2] = "kIntersectMode";
        ShapeMode[ShapeMode["kDifferenceMode"] = 3] = "kDifferenceMode";
        ShapeMode[ShapeMode["kNoneMode"] = 4] = "kNoneMode";
    })(ShapeMode || (ShapeMode = {}));
    var ShapeInfo = (function () {
        function ShapeInfo() {
            this.mIndex = -1;
            this.mInverted = false;
            this.mPaths = [];
            this.mClosed = true;
            this.mMode = ShapeMode.kAddMode;
        }
        ShapeInfo.prototype.toJson = function (writer) {
            writer.startObject();
            writer.putKey("index");
            writer.putNumber(this.mIndex);
            writer.putKey("invert");
            writer.putBoolean(this.mInverted);
            writer.putKey("mode");
            writer.putNumber(this.mMode);
            writer.putKey("closed");
            writer.putBoolean(this.mClosed);
            writer.putKey("paths");
            writer.startArray();
            for (var p = 0; p < this.mPaths.length; p++) {
                this.mPaths[p].toJson(writer);
            }
            writer.endArray();
            writer.endObject();
        };
        ShapeInfo.prototype.fromJson = function (data) {
            this.mPaths = [];
            this.mClosed = true;
            this.mInverted = false;
            this.mMode = ShapeMode.kAddMode;
            var index = data.index;
            if (typeof (index) != "undefined") {
                this.mIndex = index;
            }
            var closed = data.closed;
            if (typeof (closed) != "undefined") {
                this.mClosed = closed;
            }
            var invert = data.invert;
            if (typeof (invert) != "undefined") {
                this.mInverted = invert;
            }
            var mode = data.mode;
            if (typeof (mode) != "undefined") {
                this.mMode = mode;
            }
            var paths = data.paths;
            if (paths instanceof Array) {
                for (var i = 0; i < paths.length; i++) {
                    var path = new PathInfo();
                    path.fromJson(paths[i]);
                    this.mPaths.push(path);
                }
            }
        };
        return ShapeInfo;
    }());
    var ResultInfo = (function () {
        function ResultInfo(x, y, w, h) {
            this.mOriginX = x;
            this.mOriginY = y;
            this.mWidth = w;
            this.mHeight = h;
            this.mIndices = [];
            this.clear();
        }
        ResultInfo.prototype.clear = function () {
            this.mShapes = [];
            this.mIndices = [];
            this.mLastPoint = null;
            this.mLastHandle = null;
        };
        ResultInfo.prototype.moveTo = function (x, y) {
            var shape = new ShapeInfo();
            this.mShapes.push(shape);
            this.mLastPoint = new PointInfo(x, y);
            this.mLastHandle = null;
        };
        ResultInfo.prototype.lineTo = function (x, y) {
            if (this.mShapes.length > 0) {
                if (this.mLastPoint) {
                    var path = new PathInfo();
                    path.mPoint1 = this.mLastPoint;
                    path.mPoint2 = new PointInfo(x, y);
                    this.mShapes[this.mShapes.length - 1].mPaths.push(path);
                }
                this.mLastPoint = new PointInfo(x, y);
                this.mLastHandle = null;
            }
        };
        ResultInfo.prototype.bezierTo = function (x, y, cx1, cy1, cx2, cy2) {
            if (this.mShapes.length > 0) {
                if (this.mLastPoint) {
                    var path = new PathInfo();
                    path.mPoint1 = this.mLastPoint;
                    path.mPoint2 = new PointInfo(x, y);
                    path.mHandle1 = new PointInfo(cx1, cy1);
                    path.mHandle2 = new PointInfo(cx2, cy2);
                    this.mShapes[this.mShapes.length - 1].mPaths.push(path);
                }
                this.mLastPoint = new PointInfo(x, y);
                this.mLastHandle = null;
            }
        };
        ResultInfo.prototype.relativeBezierTo = function (x, y, cx1, cy1, cx2, cy2) {
            if (this.mShapes.length > 0) {
                if (this.mLastPoint) {
                    var path = new PathInfo();
                    path.mPoint1 = this.mLastPoint;
                    path.mPoint2 = new PointInfo(x, y);
                    path.mHandle1 = new PointInfo(cx1 + this.mLastPoint.mX, cy1 + this.mLastPoint.mY);
                    path.mHandle2 = new PointInfo(cx2 + x, cy2 + y);
                    this.mShapes[this.mShapes.length - 1].mPaths.push(path);
                }
                this.mLastPoint = new PointInfo(x, y);
                this.mLastHandle = null;
            }
        };
        ResultInfo.prototype.lastShape = function () {
            if (this.mShapes.length > 0) {
                return this.mShapes[this.mShapes.length - 1];
            }
            return null;
        };
        ResultInfo.prototype.toJson = function () {
            var writer = new JsonWriter();
            writer.startObject();
            writer.putKey("shapes");
            writer.startArray();
            for (var s = 0, l = this.mShapes.length; s < l; s++) {
                this.mShapes[s].toJson(writer);
            }
            writer.endArray();
            writer.putKey("indices");
            writer.startArray();
            for (var s = 0, l = this.mIndices.length; s < l; s++) {
                writer.putNumber(this.mIndices[s]);
            }
            writer.endArray();
            writer.putKey("x");
            writer.putNumber(this.mOriginX);
            writer.putKey("y");
            writer.putNumber(this.mOriginY);
            writer.putKey("width");
            writer.putNumber(this.mWidth);
            writer.putKey("height");
            writer.putNumber(this.mHeight);
            writer.endObject();
            return writer.getContent();
        };
        ResultInfo.prototype.fromJson = function (json) {
            this.clear();
            var data = JSON.parse(json);
            var shapes = data.shapes;
            if (shapes instanceof Array) {
                for (var s = 0; s < shapes.length; s++) {
                    var shape = new ShapeInfo();
                    shape.fromJson(shapes[s]);
                    this.mShapes.push(shape);
                }
            }
            var indices = data.indices;
            if (typeof (indices) != "undefined") {
                this.mIndices = indices;
            }
            var x = data.x;
            if (typeof (x) != "undefined") {
                this.mOriginX = x;
            }
            var y = data.y;
            if (typeof (y) != "undefined") {
                this.mOriginY = y;
            }
            var width = data.width;
            if (typeof (width) != "undefined") {
                this.mWidth = width;
            }
            var height = data.height;
            if (typeof (height) != "undefined") {
                this.mHeight = height;
            }
        };
        return ResultInfo;
    }());
    var BooleanOperation;
    (function (BooleanOperation) {
        BooleanOperation["kUniteOperation"] = "unite";
        BooleanOperation["kIntersectOperation"] = "intersect";
        BooleanOperation["kSubtractOperation"] = "subtract";
        BooleanOperation["kExcludeOperation"] = "exclude";
        BooleanOperation["kDivideOperation"] = "divide";
    })(BooleanOperation || (BooleanOperation = {}));
    var ShapeContext = (function () {
        function ShapeContext() {
        }
        ShapeContext.Setup = function (canvas) {
            ShapeContext.kPaper = new paper.PaperScope();
            ShapeContext.kPaper.setup(canvas);
        };
        ShapeContext.ToPaperPoint = function (point) {
            return new paper.Point(point.mX, point.mY);
        };
        ShapeContext.ToPlainSimplePaperPath = function (shape) {
            var result = new paper.Path();
            if (shape.mPaths.length > 1) {
                result.moveTo(ShapeContext.ToPaperPoint(shape.mPaths[0].mPoint1));
            }
            for (var p = 0, l = shape.mPaths.length; p < l; p++) {
                var path = shape.mPaths[p];
                if (path.mHandle1 != null) {
                    result.cubicCurveTo(ShapeContext.ToPaperPoint(path.mHandle1), ShapeContext.ToPaperPoint(path.mHandle2), ShapeContext.ToPaperPoint(path.mPoint2));
                }
                else {
                    result.lineTo(ShapeContext.ToPaperPoint(path.mPoint2));
                }
            }
            if (shape.mClosed) {
                result.closePath();
            }
            return result;
        };
        ShapeContext.ToSimplePaperPath = function (shape, boundsPath) {
            var result = new paper.Path();
            if (shape.mPaths.length > 1) {
                result.moveTo(ShapeContext.ToPaperPoint(shape.mPaths[0].mPoint1));
            }
            for (var p = 0, l = shape.mPaths.length; p < l; p++) {
                var path = shape.mPaths[p];
                if (path.mHandle1 != null) {
                    result.cubicCurveTo(ShapeContext.ToPaperPoint(path.mHandle1), ShapeContext.ToPaperPoint(path.mHandle2), ShapeContext.ToPaperPoint(path.mPoint2));
                }
                else {
                    result.lineTo(ShapeContext.ToPaperPoint(path.mPoint2));
                }
            }
            if (shape.mClosed) {
                result.closePath();
                if (shape.mInverted && boundsPath != null) {
                    var tmp = void 0;
                    tmp = boundsPath.subtract(result);
                    result.remove();
                    return tmp;
                }
            }
            return result;
        };
        ShapeContext.ToMultiplePaperPath = function (info, holder) {
            for (var s = 0; s < info.mShapes.length; s++) {
                var shape = info.mShapes[s];
                holder.push(ShapeContext.ToSimplePaperPath(shape, null));
            }
        };
        ShapeContext.OperatePaths = function (info, operation) {
            if (info.mShapes.length > 0) {
                var path = ShapeContext.ToSimplePaperPath(info.mShapes[0], null);
                for (var s = 1; s < info.mShapes.length; s++) {
                    var shape = info.mShapes[s];
                    var current = ShapeContext.ToSimplePaperPath(shape, null);
                    var tmp = null;
                    switch (operation) {
                        case BooleanOperation.kUniteOperation:
                            tmp = path.unite(current);
                            break;
                        case BooleanOperation.kIntersectOperation:
                            tmp = path.intersect(current);
                            break;
                        case BooleanOperation.kSubtractOperation:
                            tmp = path.subtract(current);
                            break;
                        case BooleanOperation.kExcludeOperation:
                            tmp = path.exclude(current);
                            break;
                        case BooleanOperation.kDivideOperation:
                            tmp = path.divide(current);
                            break;
                        default:
                            break;
                    }
                    current.remove();
                    path.remove();
                    path = tmp;
                }
                return path;
            }
            return null;
        };
        ShapeContext.SmoothPaths = function (info, output, smoothFactor) {
            var option = { type: 'catmull-rom', factor: smoothFactor };
            if (info.mShapes.length > 0) {
                for (var s = 0; s < info.mShapes.length; s++) {
                    var path = ShapeContext.ToPlainSimplePaperPath(info.mShapes[s]);
                    path.smooth(option);
                    if (ShapeContext.FromPaperPath(output, path)) {
                        output.lastShape().mMode = info.mShapes[s].mMode;
                        output.lastShape().mIndex = info.mShapes[0].mIndex + s;
                    }
                }
            }
        };
        ShapeContext.SimplifyPaths = function (info, output, tolerance) {
            if (info.mShapes.length > 0) {
                for (var s = 0; s < info.mShapes.length; s++) {
                    var path = ShapeContext.ToPlainSimplePaperPath(info.mShapes[s]);
                    path.simplify(tolerance);
                    if (ShapeContext.FromPaperPath(output, path)) {
                        output.lastShape().mMode = info.mShapes[s].mMode;
                        output.lastShape().mIndex = info.mShapes[0].mIndex + s;
                    }
                }
            }
        };
        ShapeContext.ToComposedPaperPath = function (info) {
            if (info.mShapes.length > 0) {
                var boundsPath = new paper.Path.Rectangle(new paper.Rectangle(info.mOriginX, info.mOriginY, info.mWidth, info.mHeight));
                var path = null;
                if (info.mShapes[0].mMode == ShapeMode.kSubtractMode) {
                    var shape = info.mShapes[0];
                    var subtractEnd = ShapeContext.ToSimplePaperPath(shape, boundsPath);
                    path = boundsPath.subtract(subtractEnd);
                    subtractEnd.remove();
                }
                else {
                    var shape = info.mShapes[0];
                    path = ShapeContext.ToSimplePaperPath(shape, boundsPath);
                }
                for (var s = 1; s < info.mShapes.length; s++) {
                    var shape = info.mShapes[s];
                    var current = ShapeContext.ToSimplePaperPath(shape, boundsPath);
                    var tmp = null;
                    switch (shape.mMode) {
                        case ShapeMode.kAddMode:
                            tmp = path.unite(current);
                            break;
                        case ShapeMode.kSubtractMode:
                            tmp = path.subtract(current);
                            break;
                        case ShapeMode.kIntersectMode:
                            tmp = path.intersect(current);
                            break;
                        case ShapeMode.kDifferenceMode:
                            var united = path.unite(current);
                            var intersected = path.intersect(current);
                            tmp = united.subtract(intersected);
                            united.remove();
                            intersected.remove();
                            break;
                        default:
                            break;
                    }
                    current.remove();
                    path.remove();
                    path = tmp;
                }
                return path;
            }
            return null;
        };
        ShapeContext.FromPaperPath = function (info, path) {
            if (path.children == null && path.curves.length >= 1) {
                var pp = path;
                var start = new PointInfo(pp.curves[0].point1.x, pp.curves[0].point1.y);
                info.moveTo(pp.curves[0].point1.x, pp.curves[0].point1.y);
                var l = pp.curves.length;
                for (var p = 0; p < l; p++) {
                    var curve = pp.curves[p];
                    if (!curve.isStraight()) {
                        info.relativeBezierTo(curve.point2.x, curve.point2.y, curve.handle1.x, curve.handle1.y, curve.handle2.x, curve.handle2.y);
                    }
                    else {
                        info.lineTo(curve.point2.x, curve.point2.y);
                    }
                }
                if (pp.closed && !info.mLastPoint.equal(start)) {
                    info.lineTo(start.mX, start.mY);
                }
                info.lastShape().mClosed = pp.closed;
                return true;
            }
            else if (path.children && path.children.length > 0) {
                var ret = false;
                for (var c = 0; c < path.children.length; c++) {
                    var pp = path.children[c];
                    ret = this.FromPaperPath(info, pp) || ret;
                }
                return ret;
            }
        };
        ShapeContext.FromPaperPaths = function (info, paths) {
            if (paths.length >= 1) {
                for (var c = 0; c < paths.length; c++) {
                    this.FromPaperPath(info, paths[c]);
                }
                return true;
            }
            return false;
        };
        ShapeContext.FromCompoundPaperPath = function (info, path) {
            info.mShapes = [];
            info.mLastPoint = null;
            info.mLastHandle = null;
            for (var p = 0, l = path.children.length; p < l; p++) {
                var subpath = path.children[p];
                ShapeContext.FromPaperPath(info, subpath);
                info.lastShape().mMode = ShapeMode.kDifferenceMode;
            }
        };
        ShapeContext.kPaper = null;
        return ShapeContext;
    }());
    function Setup(canvas) {
        ShapeContext.Setup(canvas);
    }
    MaskHelper.Setup = Setup;
    function PathFinder(cmd, operation) {
        var inputInfo = new ResultInfo(0, 0, 0, 0);
        inputInfo.fromJson(cmd);
        var output = ShapeContext.OperatePaths(inputInfo, operation);
        if (output) {
            var resultInfo = new ResultInfo(0, 0, 0, 0);
            resultInfo.mIndices = inputInfo.mIndices;
            ShapeContext.FromPaperPath(resultInfo, output);
            for (var i = 0; i < resultInfo.mShapes.length; i++) {
                resultInfo.mShapes[i].mIndex = inputInfo.mShapes[0].mIndex + i;
            }
            return resultInfo;
        }
        return null;
    }
    MaskHelper.PathFinder = PathFinder;
    function PathSmooth(cmd, factor) {
        var inputInfo = new ResultInfo(0, 0, 0, 0);
        inputInfo.fromJson(cmd);
        var resultInfo = new ResultInfo(0, 0, 0, 0);
        resultInfo.mIndices = inputInfo.mIndices;
        ShapeContext.SmoothPaths(inputInfo, resultInfo, factor);
        return resultInfo;
    }
    MaskHelper.PathSmooth = PathSmooth;
    function PathSimplify(cmd, tolerance) {
        var inputInfo = new ResultInfo(0, 0, 0, 0);
        inputInfo.fromJson(cmd);
        var resultInfo = new ResultInfo(0, 0, 0, 0);
        resultInfo.mIndices = inputInfo.mIndices;
        ShapeContext.SimplifyPaths(inputInfo, resultInfo, tolerance);
        return resultInfo;
    }
    MaskHelper.PathSimplify = PathSimplify;
})(MaskHelper || (MaskHelper = {}));
