"use strict";
var MHInterface = (function(){
	function MHInterface() {		
	}

	MHInterface.updateThemeWithAppSkinInfo = function(){
		function addStyleRule(selector, rule) {
			var createSheet = function () {
				var style = document.createElement("style");
				document.head.appendChild(style);
				return style.sheet||style.styleSheet;
			};
			var sheet = createSheet();
			if( sheet.addRule )
			{
			   sheet.addRule(selector, rule);
			}  else if( sheet.insertRules) {
			   sheet.insertRules(selector + ' { ' + rule + ' }', sheet.cssRules.length);
			}
		}

		function toHex(color, delta) {
			function computeValue(value, delta) {
				var computedValue = !isNaN(delta) ? value + delta : value;
				if (computedValue < 0) {
					computedValue = 0;
				} else if (computedValue > 255) {
					computedValue = 255;
				}
				computedValue = Math.round(computedValue).toString(16);
				return computedValue.length == 1 ? "0" + computedValue : computedValue;
			}
			var hex = "";
			if (color) {
				hex = computeValue(color.red, delta) + computeValue(color.green, delta) + computeValue(color.blue, delta);
			}
			return "#" + hex;
		}
		
		function updateThemeWithAppSkinInfoPrivate(appSkinInfo) {
			var panelBackgroundColor = appSkinInfo.panelBackgroundColor.color;
			var isPanelThemeLight = panelBackgroundColor.red > 127;
			var fontColor, disabledFontColor;
			var borderColor;
			var inputBackgroundColor;
			var gradientHighlightBg;
			if(isPanelThemeLight) {
				fontColor = "#000000;";
				disabledFontColor = "color:" + toHex(panelBackgroundColor, -70) + ";";
				borderColor = toHex(panelBackgroundColor, -90);
				inputBackgroundColor = toHex(panelBackgroundColor, 54);
				gradientHighlightBg = "background-image: -webkit-linear-gradient(top, " + toHex(panelBackgroundColor, -40) + " , " + toHex(panelBackgroundColor,-50) + ");";
			} else {
				fontColor = "#A5A5A5;";
				disabledFontColor = "color:" + toHex(panelBackgroundColor, 100) + ";";
				borderColor = toHex(panelBackgroundColor, -45);
				inputBackgroundColor = toHex(panelBackgroundColor, -20);
				gradientHighlightBg = "background-image: -webkit-linear-gradient(top, " + toHex(panelBackgroundColor, -20) + " , " + toHex(panelBackgroundColor, -30) + ");";
			}

			addStyleRule(".ae-bg","background-color:" + toHex(panelBackgroundColor) + "; color:" + fontColor )
			addStyleRule(".panel-body", "padding:5px;")
			addStyleRule(".panel-default>.panel-heading","background-color:" + toHex(panelBackgroundColor, -10) + "; color:" + fontColor + "; border-color:" + toHex(panelBackgroundColor, -10) + ";height:26px; text-align:center; padding:2px 5px;border-top-left-radius: 0px; border-top-right-radius: 0px;" )
			addStyleRule(".panel-default","border-color:" + toHex(panelBackgroundColor, -20) + ";margin-bottom: 10px;border-radius: 4px;" )
			addStyleRule(".dropdown-toggle","background-color:" + toHex(panelBackgroundColor, -10) + "; color:" + fontColor + "; padding-top:2px; padding-bottom:2px;" )
			addStyleRule(".open>.dropdown-menu","background-color:" + toHex(panelBackgroundColor, -10) + "; color:" + fontColor )
			addStyleRule(".dropdown-menu>li>a","background-color:" + toHex(panelBackgroundColor, -10) + "; color:" + fontColor )
			addStyleRule(".btn.focus, .btn:focus, .btn:hover","color:" + fontColor )
			addStyleRule(".input-group-addon","border:none; background-color:" + toHex(panelBackgroundColor, -10) + "; color:" + fontColor )
			addStyleRule(".form-control","border:none; background-color:" + inputBackgroundColor + "; color:" + fontColor )
			addStyleRule(".btn-group-sm>.btn", "background-color:" + toHex(panelBackgroundColor, -10) + "; border-color:" + borderColor + ";padding:10px 10px;border-color:" + toHex(panelBackgroundColor, -20) + ";color:"+fontColor+";")
			addStyleRule(".btn-group-sm>.btn:active", "background-color:" + toHex(panelBackgroundColor, -20) + "; border-color:" + borderColor + ";padding:10px 10px;border-color:" + toHex(panelBackgroundColor, -20) + ";")
			addStyleRule(".panel-body>.btn-default", "background-color:" + toHex(panelBackgroundColor, -10) + "; border-color:" + borderColor + ";padding:10px 10px;border-color:" + toHex(panelBackgroundColor, -20) + ";color:"+fontColor+";")
			addStyleRule(".panel-body>.btn-default:active", "background-color:" + toHex(panelBackgroundColor, -20) + "; border-color:" + borderColor + ";padding:10px 10px;border-color:" + toHex(panelBackgroundColor, -20) + ";color:" + toHex(panelBackgroundColor, 10) + ";")
			addStyleRule(".panel-body>.btn-default:hover", "background-color:" + toHex(panelBackgroundColor, -20) + "; border-color:" + borderColor + ";padding:10px 10px;border-color:" + toHex(panelBackgroundColor, -20) + ";")
			addStyleRule(".ss-slider-holder", "margin-bottom:5px;margin-top:5px;")
			addStyleRule(".slider.slider-horizontal", "width:100%")
			addStyleRule(".well", "margin-bottom: 0px;")
			addStyleRule(".slider-track","background:"+toHex(panelBackgroundColor)+";")
			addStyleRule(".slider-selection","background:" + toHex(panelBackgroundColor, -20) + ";")
			addStyleRule(".slider-track-high","background:" + toHex(panelBackgroundColor, -15) + ";")
			addStyleRule(".slider-handle","background:" + toHex(panelBackgroundColor, -13) + ";")
			addStyleRule(".well", "background-color:" + toHex(panelBackgroundColor, -5) + "; border-color:" + borderColor + "; padding-top:2px; padding-bottom:2px;")
		}

		function onAppThemeColorChanged(event) {
			var skinInfo = JSON.parse(window.__adobe_cep__.getHostEnvironment()).appSkinInfo;
			updateThemeWithAppSkinInfoPrivate(skinInfo);
		}
		updateThemeWithAppSkinInfoPrivate(MHInterface.cef.hostEnvironment.appSkinInfo);
	};

	MHInterface.escapeString = function(str){
		return (str + '').replace(/[\\"']/g, '\\$&').replace(/\u0000/g, '\\0');
	}
	
	MHInterface.getHostScript = function(name){
		return MHInterface.cef.getSystemPath(SystemPath.EXTENSION) + "/jsx/" + name;
	}
	
	MHInterface.init = function(){
		MHInterface.cef = new CSInterface();
		MHInterface.updateThemeWithAppSkinInfo()
		MHInterface.locale = MHInterface.cef.initResourceBundle();
		MHInterface.cef.evalScript('$.evalFile("'+ MHInterface.getHostScript('Entry.jsx') +'")', function(){});
		MaskHelper.Setup(document.getElementById("viewer"))
	}

	MHInterface.pathFinder = function(operation){
		MHInterface.cef.evalScript("MaskHelperAE.FetchLayerSelectedMasks('')", function(fetchRet){
			var cmd = JSON.parse(fetchRet)
			if(typeof(cmd.error) != "undefined" && cmd.error != null){
				if(cmd.error == "animated"){
					MHInterface.alert(MHInterface.locale.UI_AnimatedMaskError)
				}else{
					MHInterface.alert(MHInterface.locale.UI_ClosedMaskLessThan2)
				}
			} else {
				if(cmd.shapes == null || cmd.shapes.length < 2){
					MHInterface.alert(MHInterface.locale.UI_ClosedMaskLessThan2)
				} else {
					var result = MaskHelper.PathFinder(fetchRet, operation)
					if(result != null){
						var resultStr = result.toJson()
						MHInterface.cef.evalScript('MaskHelperAE.GenerateShapeResult("' + MHInterface.escapeString(resultStr) + '")', function(ret){
						})
					}
				}
			}
		})
	}

	MHInterface.pathSmooth = function(){
		MHInterface.cef.evalScript("MaskHelperAE.FetchLayerSelectedMasks('acceptOpenPaths')", function(fetchRet){
			var cmd = JSON.parse(fetchRet)
			if(typeof(cmd.error) != "undefined" && cmd.error != null){
				if(cmd.error == "animated"){
					MHInterface.alert(MHInterface.locale.UI_AnimatedMaskError)
				}else{
					MHInterface.alert(MHInterface.locale.UI_MaskLessThan2)
				}
			} else {
				var result = MaskHelper.PathSmooth(fetchRet, MHInterface.smoothFactor)
				if(result != null){
					var resultStr = result.toJson()
					MHInterface.cef.evalScript('MaskHelperAE.GenerateShapeResult("' + MHInterface.escapeString(resultStr) + '")', function(ret){ })
				}
			}
		})
	}
	
	MHInterface.pathSimplify = function(){
		MHInterface.cef.evalScript("MaskHelperAE.FetchLayerSelectedMasks('acceptOpenPaths')", function(fetchRet){
			var cmd = JSON.parse(fetchRet)
			if(typeof(cmd.error) != "undefined" && cmd.error != null){
				if(cmd.error == "animated"){
					MHInterface.alert(MHInterface.locale.UI_AnimatedMaskError)
				}else{
					MHInterface.alert(MHInterface.locale.UI_MaskLessThan2)
				}
			} else {
				var result = MaskHelper.PathSimplify(fetchRet, MHInterface.simplifyTolerance)
				if(result != null){
					var resultStr = result.toJson()
					MHInterface.cef.evalScript('MaskHelperAE.GenerateShapeResult("' + MHInterface.escapeString(resultStr) + '")', function(ret){})
				}
			}
		})
	}

	MHInterface.alert = function(msg, title){
		title = title || "VE Mask Helper"
		MHInterface.cef.evalScript('MaskHelperAE.Alert(' + JSON.stringify(msg) + ',' + JSON.stringify(title) +')', function(){
		});
	}

	MHInterface.showModal = function(title, content){
		if (typeof(title) != "undefined" &&  typeof(content) != "undefined") {
			document.getElementById('modal-title').innerHTML = title
			document.getElementById('modal-text').innerHTML = content
		}else{
			document.getElementById('modal-title').innerHTML = "Processing"
			document.getElementById('modal-text').innerHTML = "Task processing, don't touch anything ..."
		}
		document.getElementById('modal').style.display = "block";
	}

	MHInterface.hideModal = function(){
		document.getElementById('modal').style.display = "none";
	}

	$(document).ready(function(){
		MHInterface.init();
		$("#path-smooth").on("slide", function(slideEvt) {
			MHInterface.smoothFactor = slideEvt.value / 100.0
		});
		$("#path-simplify").on("slide", function(slideEvt) {
			MHInterface.simplifyTolerance = slideEvt.value
		});
	});

	MHInterface.cef = null
	MHInterface.locale = null
	MHInterface.smoothFactor = 0.5
	MHInterface.simplifyTolerance = 2.5
	return MHInterface;
}());






